@@ -5,16 +5,16 @@ pub use buffered_file::BufferedFile;
 use {
     super::string::format,
     crate::{
-        runtime_error, ExternalData, ExternalValue, KotoFile, KotoRead, KotoWrite, MetaMap, Mutex,
-        RuntimeError, RwLock, Value, ValueMap, Vm,
+        error::unexpected_type_error_with_slice, runtime_error, ExternalData, ExternalValue,
+        KotoFile, KotoRead, KotoWrite, MetaMap, RuntimeError, Value, ValueMap, Vm,
     },
-    lazy_static::lazy_static,
     std::{
+        cell::RefCell,
         fmt, fs,
         io::{self, BufRead, Read, Seek, SeekFrom, Write},
         ops::Deref,
         path::{Path, PathBuf},
-        sync::Arc,
+        rc::Rc,
     },
 };
 
@@ -34,11 +34,11 @@ pub fn make_module() -> ValueMap {
                     }
                 }
             }
-            [unexpected] => runtime_error!(
-                "io.create: Expected a String as argument, found '{}'",
-                unexpected.type_as_string(),
+            unexpected => unexpected_type_error_with_slice(
+                "io.create",
+                "a path String as argument",
+                unexpected,
             ),
-            _ => runtime_error!("io.create: Expected a String as argument"),
         }
     });
 
@@ -52,7 +52,9 @@ pub fn make_module() -> ValueMap {
 
     result.add_fn("exists", |vm, args| match vm.get_args(args) {
         [Str(path)] => Ok(Bool(fs::canonicalize(path.as_str()).is_ok())),
-        _ => runtime_error!("io.exists: Expected path string as argument"),
+        unexpected => {
+            unexpected_type_error_with_slice("io.exists", "a path String as argument", unexpected)
+        }
     });
 
     result.add_fn("extend_path", |vm, args| match vm.get_args(args) {
@@ -66,11 +68,15 @@ pub fn make_module() -> ValueMap {
             }
             Ok(path.to_string_lossy().to_string().into())
         }
-        _ => runtime_error!("io.extend_path: Expected path string as first argument"),
+        unexpected => unexpected_type_error_with_slice(
+            "io.extend_path",
+            "a path String as argument, followed by some additional path nodes",
+            unexpected,
+        ),
     });
 
     result.add_fn("open", {
-        move |vm, args| match vm.get_args(args) {
+        |vm, args| match vm.get_args(args) {
             [Str(path)] => match fs::canonicalize(path.as_str()) {
                 Ok(path) => match fs::File::open(&path) {
                     Ok(file) => Ok(File::system_file(file, path)),
@@ -80,11 +86,9 @@ pub fn make_module() -> ValueMap {
                 },
                 Err(_) => runtime_error!("io.open: Failed to canonicalize path"),
             },
-            [unexpected] => runtime_error!(
-                "io.open: Expected a String as argument, found '{}'",
-                unexpected.type_as_string(),
-            ),
-            _ => runtime_error!("io.open: Expected a String as argument"),
+            unexpected => {
+                unexpected_type_error_with_slice("io.open", "a path String as argument", unexpected)
+            }
         }
     });
 
@@ -100,12 +104,18 @@ pub fn make_module() -> ValueMap {
                     Err(error) => Err(error),
                 }
             }
-            _ => return runtime_error!("io.print: Expected a string as format argument"),
+            unexpected => {
+                return unexpected_type_error_with_slice(
+                    "io.print",
+                    "a String as argument, followed by optional additional Values",
+                    unexpected,
+                )
+            }
         };
 
         match result {
             Ok(_) => Ok(Empty),
-            Err(e) => Err(e.with_prefix("string.print")),
+            Err(e) => Err(e.with_prefix("io.print")),
         }
     });
 
@@ -114,7 +124,11 @@ pub fn make_module() -> ValueMap {
             Ok(result) => Ok(Str(result.into())),
             Err(e) => runtime_error!("io.read_to_string: Unable to read file '{}': {}", path, e),
         },
-        _ => runtime_error!("io.read_to_string: Expected path string as argument"),
+        unexpected => unexpected_type_error_with_slice(
+            "io.read_to_string",
+            "a path String as argument",
+            unexpected,
+        ),
     });
 
     result.add_fn("remove_file", {
@@ -130,11 +144,11 @@ pub fn make_module() -> ValueMap {
                     ),
                 }
             }
-            [unexpected] => runtime_error!(
-                "io.remove_file: Expected a String as argument, found '{}'",
-                unexpected.type_as_string(),
+            unexpected => unexpected_type_error_with_slice(
+                "io.remove_file",
+                "a path String as argument",
+                unexpected,
             ),
-            _ => runtime_error!("io.remove_file: Expected a String as argument"),
         }
     });
 
@@ -149,8 +163,8 @@ pub fn make_module() -> ValueMap {
     result
 }
 
-lazy_static! {
-    pub static ref FILE_META: Arc<RwLock<MetaMap>> = {
+thread_local!(
+    pub static FILE_META: Rc<RefCell<MetaMap>> = {
         use Value::{Empty, Number, Str};
 
         let mut meta = MetaMap::with_type_name("File");
@@ -193,11 +207,11 @@ lazy_static! {
                     Err(e) => Err(e.with_prefix("File.seek")),
                 }
             }
-            [unexpected] => runtime_error!(
-                "File.seek: Expected Number for seek position, found '{}'",
-                unexpected.type_as_string(),
+            unexpected => unexpected_type_error_with_slice(
+                "File.seek",
+                "a non-negative Number as the seek position",
+                unexpected,
             ),
-            _ => runtime_error!("File.seek: Expected seek position as second argument"),
         });
 
         meta.add_named_instance_fn_mut("write", |file: &mut File, _, args| match args {
@@ -205,16 +219,22 @@ lazy_static! {
                 Ok(_) => Ok(Value::Empty),
                 Err(e) => Err(e.with_prefix("File.write")),
             },
-            _ => runtime_error!("File.write: Expected single value to write as argument"),
+            unexpected => unexpected_type_error_with_slice(
+                "File.write",
+                "a single argument",
+                unexpected,
+            ),
         });
 
         meta.add_named_instance_fn_mut("write_line", |file: &mut File, _, args| {
             let line = match args {
                 [] => "\n".to_string(),
                 [value] => format!("{}\n", value),
-                _ => {
-                    return runtime_error!("File.write_line: Expected single value as argument");
-                }
+                unexpected => return unexpected_type_error_with_slice(
+                    "File.write_line",
+                    "a single argument",
+                    unexpected,
+                ),
             };
             match file.write(line.as_bytes()) {
                 Ok(_) => Ok(Value::Empty),
@@ -222,15 +242,15 @@ lazy_static! {
             }
         });
 
-        Arc::new(RwLock::new(meta))
-    };
-}
+        Rc::new(RefCell::new(meta))
+    }
+);
 
 /// The File type used in the io module
-pub struct File(Arc<dyn KotoFile>);
+pub struct File(Rc<dyn KotoFile>);
 
 impl Deref for File {
-    type Target = Arc<dyn KotoFile>;
+    type Target = Rc<dyn KotoFile>;
 
     fn deref(&self) -> &Self::Target {
         &self.0
@@ -241,32 +261,33 @@ impl File {
     /// Wraps a file that implements traits typical of a system file in a buffered reader/writer
     pub fn system_file<T>(file: T, path: PathBuf) -> Value
     where
-        T: Read + Write + Seek + Send + Sync + 'static,
+        T: Read + Write + Seek + 'static,
     {
         let result = ExternalValue::with_shared_meta_map(
-            File(Arc::new(BufferedSystemFile::new(file, path))),
-            FILE_META.clone(),
+            Self(Rc::new(BufferedSystemFile::new(file, path))),
+            Self::meta(),
         );
         Value::ExternalValue(result)
     }
 
     fn stderr(vm: &Vm) -> Value {
-        let result =
-            ExternalValue::with_shared_meta_map(File(vm.stderr().clone()), FILE_META.clone());
+        let result = ExternalValue::with_shared_meta_map(Self(vm.stderr().clone()), Self::meta());
         Value::ExternalValue(result)
     }
 
     fn stdin(vm: &Vm) -> Value {
-        let result =
-            ExternalValue::with_shared_meta_map(File(vm.stdin().clone()), FILE_META.clone());
+        let result = ExternalValue::with_shared_meta_map(Self(vm.stdin().clone()), Self::meta());
         Value::ExternalValue(result)
     }
 
     fn stdout(vm: &Vm) -> Value {
-        let result =
-            ExternalValue::with_shared_meta_map(File(vm.stdout().clone()), FILE_META.clone());
+        let result = ExternalValue::with_shared_meta_map(Self(vm.stdout().clone()), Self::meta());
         Value::ExternalValue(result)
     }
+
+    fn meta() -> Rc<RefCell<MetaMap>> {
+        FILE_META.with(|meta| meta.clone())
+    }
 }
 
 impl ExternalData for File {
@@ -291,17 +312,17 @@ struct BufferedSystemFile<T>
 where
     T: Write,
 {
-    file: Mutex<BufferedFile<T>>,
+    file: RefCell<BufferedFile<T>>,
     path: PathBuf,
 }
 
 impl<T> BufferedSystemFile<T>
 where
-    T: Read + Write + Seek + Send + Sync,
+    T: Read + Write + Seek,
 {
     pub fn new(file: T, path: PathBuf) -> Self {
         Self {
-            file: Mutex::new(BufferedFile::new(file)),
+            file: RefCell::new(BufferedFile::new(file)),
             path,
         }
     }
@@ -309,7 +330,7 @@ where
 
 impl<T> KotoFile for BufferedSystemFile<T>
 where
-    T: Read + Write + Seek + Send + Sync,
+    T: Read + Write + Seek,
 {
     fn path(&self) -> Result<String, RuntimeError> {
         Ok(self.path.to_string_lossy().into())
@@ -317,7 +338,7 @@ where
 
     fn seek(&self, position: u64) -> Result<(), RuntimeError> {
         self.file
-            .lock()
+            .borrow_mut()
             .seek(SeekFrom::Start(position))
             .map_err(map_io_err)?;
         Ok(())
@@ -332,7 +353,7 @@ where
         let mut buffer = String::new();
         match self
             .file
-            .lock()
+            .borrow_mut()
             .read_line(&mut buffer)
             .map_err(map_io_err)?
         {
@@ -344,7 +365,7 @@ where
     fn read_to_string(&self) -> Result<String, RuntimeError> {
         let mut buffer = String::new();
         self.file
-            .lock()
+            .borrow_mut()
             .read_to_string(&mut buffer)
             .map_err(map_io_err)?;
         Ok(buffer)
@@ -356,19 +377,19 @@ where
     T: Read + Write,
 {
     fn write(&self, bytes: &[u8]) -> Result<(), RuntimeError> {
-        self.file.lock().write(bytes).map_err(map_io_err)?;
+        self.file.borrow_mut().write(bytes).map_err(map_io_err)?;
         Ok(())
     }
 
     fn write_line(&self, text: &str) -> Result<(), RuntimeError> {
-        let mut locked = self.file.lock();
-        locked.write(text.as_bytes()).map_err(map_io_err)?;
-        locked.write("\n".as_bytes()).map_err(map_io_err)?;
+        let mut borrowed = self.file.borrow_mut();
+        borrowed.write(text.as_bytes()).map_err(map_io_err)?;
+        borrowed.write("\n".as_bytes()).map_err(map_io_err)?;
         Ok(())
     }
 
     fn flush(&self) -> Result<(), RuntimeError> {
-        self.file.lock().flush().map_err(map_io_err)
+        self.file.borrow_mut().flush().map_err(map_io_err)
     }
 }
 
