@@ -1,4 +1,4 @@
-use crate::{runtime_error, Value, ValueMap, ValueTuple};
+use crate::{unexpected_type_error_with_slice, Value, ValueMap, ValueTuple};
 
 pub fn make_module() -> ValueMap {
     use Value::*;
@@ -16,7 +16,9 @@ pub fn make_module() -> ValueMap {
 
     result.add_fn("type", |vm, args| match vm.get_args(args) {
         [value] => Ok(Str(value.type_as_string().into())),
-        _ => runtime_error!("koto.type: Expected single argument"),
+        unexpected => {
+            unexpected_type_error_with_slice("koto.type", "a single argument", unexpected)
+        }
     });
 
     result
