@@ -1,9 +1,9 @@
-use {koto_bytecode::Chunk, std::sync::Arc};
+use {koto_bytecode::Chunk, std::rc::Rc};
 
 #[derive(Clone, Debug)]
 pub(crate) struct Frame {
     // The chunk being interpreted in this frame
-    pub chunk: Arc<Chunk>,
+    pub chunk: Rc<Chunk>,
     // The index in the VM value stack of the first argument register,
     // or the first local register if there are no arguments.
     pub register_base: usize,
@@ -18,7 +18,7 @@ pub(crate) struct Frame {
 }
 
 impl Frame {
-    pub fn new(chunk: Arc<Chunk>, register_base: usize) -> Self {
+    pub fn new(chunk: Rc<Chunk>, register_base: usize) -> Self {
         Self {
             chunk,
             register_base,
