@@ -3,7 +3,7 @@
 use {
     koto_bytecode::Chunk,
     koto_runtime::{num2, num4, BinaryOp, Loader, Value, Value::*, ValueList, Vm},
-    std::sync::Arc,
+    std::rc::Rc,
 };
 
 pub fn test_script(script: &str, expected_output: Value) {
@@ -48,7 +48,7 @@ pub fn test_script_with_vm(mut vm: Vm, script: &str, expected_output: Value) {
     }
 }
 
-pub fn print_chunk(script: &str, chunk: Arc<Chunk>) {
+pub fn print_chunk(script: &str, chunk: Rc<Chunk>) {
     println!("{}\n", script);
     let script_lines = script.lines().collect::<Vec<_>>();
 
@@ -70,11 +70,11 @@ where
 pub fn number_list<T>(values: &[T]) -> Value
 where
     T: Copy,
-    f64: From<T>,
+    i64: From<T>,
 {
     let values = values
         .iter()
-        .map(|n| Number(f64::from(*n).into()))
+        .map(|n| Number(i64::from(*n).into()))
         .collect::<Vec<_>>();
     value_list(&values)
 }
@@ -82,11 +82,11 @@ where
 pub fn number_tuple<T>(values: &[T]) -> Value
 where
     T: Copy,
-    f64: From<T>,
+    i64: From<T>,
 {
     let values = values
         .iter()
-        .map(|n| Number(f64::from(*n).into()))
+        .map(|n| Number(i64::from(*n).into()))
         .collect::<Vec<_>>();
     value_tuple(&values)
 }
